/**
 * @author	Sonnenspeer
 * @copyright	2013 Sonnenspeer
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 */
var Shoutbox = Class.extend({
	/**
	 * Inits Shoutbox.
	 * 
	 * @param	string		shoutboxID
	 * @param	Hash		entries
	 */
	init: function(shoutboxID, entries, time, options) {
		this.shoutboxID = shoutboxID;
		this.lastEntryID = 0;
		this.standby = false;
		this.options = $.extend(true, {
			entryReloadInterval: 	0,
			entrySortOrder: 	'ASC',
			unneededUpdateLimit:	1,
			loadAnimationStyle: 'slideDown',
			deleteAnimationStyle: 'slideUp',
			deleteEntrySure: ''
		}, options || { });
		this.unneededUpdates = 0;

		// remove entries
		var shoutboxContentDiv = $("#"+this.shoutboxID+'Content');
		if (shoutboxContentDiv.html()) {		
			//shoutboxContentDiv.html('');
			
			if (this.standby) { 
				shoutboxContentDiv.bind('mouseover', $.proxy(function() { this.stopStandby(); this.startEntryUpdate(); }, this));
			}			
		}
			
		// show smileys
		var smileyContainerDiv = $("#"+this.shoutboxID+'SmileyContainer');
		if (smileyContainerDiv.html()) {
			smileyContainerDiv.removeClass('invisible');
		}
		
		// add event listener
		var entryAddForm = $("#"+this.shoutboxID+'EntryAddForm');
		if (entryAddForm) {
			entryAddForm.bind('submit', $.proxy(function(event) { this.addEntry(); event.stopPropagation();	return false; }, this));
		}
		
		// insert entries
		this.insertEntries(entries, false);

		// start entry update
		this.startEntryUpdate();
	},
	
	/**
	 * Starts the entry update.
	 */
	startEntryUpdate: function() {
		if (this.options.entryReloadInterval != 0) {
			this.executer = new WCF.PeriodicalExecuter($.proxy(this.loadEntries, this), (this.options.entryReloadInterval * 1000));
		}
	},
	
	/**
	 * Stops the entry update.
	 */
	stopEntryUpdate: function() {
		if (this.options.entryReloadInterval != 0) {
			this.executer.stop();
		}
	},
	
	/**
	 * Starts the standby mode.
	 */
	startStandby: function() {
		if (!this.standby) {			
			this.standby = true;
			
			// change opacity
			$(this.shoutboxID+'Content').css('opacity', '0.5');
			
			// stop updating
			this.stopEntryUpdate();
		}
	},
	
	/**
	 * Stops the standby mode.
	 */
	stopStandby: function() {
		if (this.standby) {
			this.standby = false;
			this.unneededUpdates = 0;
			
			// change opacity
			$("#"+this.shoutboxID+'Content').css('opacity', '1');
			
			// start updating
			this.startEntryUpdate();
		}
	},
	
	/**
	 * Inserts a smiley with the given code.
	 * 
	 * @param	string		code
	 */
	insertSmiley: function(code) {
		var messageInputField = $("#"+this.shoutboxID+'Message');
		if (messageInputField) {
			messageInputField.val(function() { return messageInputField.val()+' '+code+' '; });
			messageInputField.focus();
		}
	},
	
	/**
	 * Adds a new entry.
	 */
	addEntry: function() {
		// get message
		var $message = '';
		var messageInputField = $("#"+this.shoutboxID+'Message');
		if (messageInputField.val()) {
			$message = messageInputField.val();
		}
		
		// get username
		var $username = '';
		var usernameInputField = $("#"+this.shoutboxID+'Username');
		if (usernameInputField.val()) {
			$username = usernameInputField.val();
		}

		// stop entry update
		this.stopEntryUpdate();
		
		// send request
		var proxyAction = new WCF.Action.Proxy({			
			success: $.proxy(function() {
				// reset message
				if (messageInputField.val()) {
					messageInputField.val("");
					messageInputField.focus();
				}
				// update entries
				this.loadEntries();
				
				// stop standby
				this.stopStandby();				
			}, this)
		});
		
		proxyAction.setOption('data', {
			actionName: 'add',
			className: 'wcf\\data\\shoutbox\\ShoutboxEntryAction',
			parameters: {
				data: {
					objectID: 0,
					username: $username,
					message: $message	
				}		
			}
		});
		proxyAction.sendRequest();
	},
	
	/**
	 * Deletes an entry.
	 * 
	 * @param	integer		id
	 */	
	deleteEntry: function(entryID) {
		// send request
		var proxyAction = new WCF.Action.Proxy({			
			success: $.proxy(function() {
				// remove entry row
				var row = $("#"+this.shoutboxID+'Entry'+entryID);
				if (row) {
					switch(this.options.deleteAnimationStyle) {
						case "bumpy":
							row.css({position: "relative"});
							row.animate({
								top: "+=3"
							}, 200);
							row.animate({
								top: "-=3",
							}, 200);
							row.animate({
								top: "+=2"
							}, 200);
							row.animate({
								top: "-=2",
							}, 200).fadeOut(200);
						break;
						case "rightLeft":
							row.css({position: "relative"});
							row.animate({
								left: "+=100"
							},500);
							row.animate({
								left: "0"
							}).fadeOut(500);
						break;
						case "slideUp":
						default:
							row.slideUp(300).fadeOut(300);
						break;
					}			
				}			
			}, this)
		});
		
		proxyAction.setOption('data', {
			actionName: 'delete',
			className: 'wcf\\data\\shoutbox\\ShoutboxEntryAction',
			parameters: {
				data: {
					objectID: 0,
					entryID: entryID
				}		
			}
		});
		proxyAction.sendRequest();
	},
	
	/**
	 * Loads the new entries and inserts them.
	 */
	loadEntries: function() {
		this.stopEntryUpdate();
		
		// start request
		var proxyAction = new WCF.Action.Proxy({		
			success: $.proxy(function(response) {
				// get entries
				var entries = response.entries;
				if (entries.length == 0) {
					this.unneededUpdates++;
					if (this.options.unneededUpdateLimit != 0 && this.unneededUpdates >= this.options.unneededUpdateLimit) {
						this.startStandby();
						return;
					}
				}
				else {
					var newEntries = new Array();
					for (var i = 0; i < entries.length; i++) {
						newEntries.push({
							entryID: entries[i].entryID,
							userID: entries[i].userID,
							username: entries[i].username,
							time: entries[i].time,
							message: entries[i].message,
							isDeletable: entries[i].isDeletable,
							userOnlineMarking: entries[i].userOnlineMarking
						});
						this.lastEntryID = entries[i].entryID;
					}
					this.unneededUpdates = 0;
					this.insertEntries(newEntries, true);
				}	
				this.startEntryUpdate();	
			}, this)
		});
		
		proxyAction.setOption('data', {
			actionName: 'loadEntries',
			className: 'wcf\\data\\shoutbox\\ShoutboxEntryAction',
			parameters: {
				data: {
					objectID: 0,
					entryID: this.lastEntryID
				}		
			}
		});
		proxyAction.sendRequest(); 
	},
	
	/**
	 * Inserts the given entries into the shoutbox.
	 * 
	 * @param	Hash		entries
	 * @param	boolean		animate
	 */
	insertEntries: function(entries, animate) {
		var shoutboxMessageDiv = $("#"+this.shoutboxID+'Content');
		if (shoutboxMessageDiv) {
			// update shoutbox content	
			if (entries.length > 0) {
				for (var key in entries) {	
						
					var id = entries[key].entryID;
					var entry = entries[key];
							
					// check if entry already exists
					if ($("#"+this.shoutboxID+'Entry'+id).html()) { this.lastEntryID = id; continue; }

					// create entry row				
					var time = $("<span/>").html('['+this._timeConverter(entry.time)+']');			
					
					var entryRow = $("<p/>", { id: this.shoutboxID+'Entry'+id }).hide();
					entryRow.append(time);
					
					if (entry.isDeletable == 1) {
						var removeLink = $("<a/>", { title: this.options.langDeleteEntry, class: "icon icon16 icon-remove-sign", onclick: "WCF.System.Confirmation.show('"+this.options.deleteEntrySure+"', $.proxy(function(action) {	if(action === 'confirm') { shoutbox.deleteEntry("+id+");	} }, this))" })
						entryRow.append(removeLink);				
					}
					entryRow.append(' ');
					
					var $username = this._sprintf(entry.userOnlineMarking, entry.username);

					if (entry.userID != 0) {		
						var userLink = $("<a/>", { href: 'index.php/User/'+entry.userID+'/' }); 
						userLink.append($username);
						entryRow.append(userLink);
					}
					else {
						entryRow.append($username);				
					}
					entryRow.append(': '+entry.message);

					// insert new entry
					if (this.options.entrySortOrder == 'ASC') {
						shoutboxMessageDiv.append(entryRow)
					}
					else {
						shoutboxMessageDiv.prepend(entryRow)
					}
					
					var shoutboxEntryDiv = $("#"+this.shoutboxID+'Entry'+id);
					if (shoutboxEntryDiv) {
						if (animate) {
							switch(this.options.loadAnimationStyle) {		
								case "fadeIn":
									entryRow.fadeIn(600);
								break;
								case "slideLeft":
									entryRow.css({ position: "relative"});
									entryRow.animate({ 
										left: '+=100' 
									}, 100).fadeIn(500);
									entryRow.animate({
										left: "0"
									}, 500);
								break;
								case "slideDown":
								default:
									entryRow.slideDown(300).fadeIn(300);
								break;
							}						
						}
						else {
							shoutboxEntryDiv.show();
						}
					}
					
					// set last entry id
					this.lastEntryID = id;
				}

				// focus last entry
				if (animate) {		
					new WCF.PeriodicalExecuter(
						$.proxy(function(executer) {
								this.focusLastEntry();
								executer.stop();
						}, 300
					), this);
				}
				else {
					this.focusLastEntry();
				}
			}
		}
	},
	
	/**
	 * Focuses the last shoutbox entry.
 	 */
	focusLastEntry: function() {
		if (this.options.entrySortOrder == 'ASC') {
			var shoutboxMessageDiv = $("#"+this.shoutboxID+'Content');
			if (shoutboxMessageDiv) {
				shoutboxMessageDiv.scrollTop = shoutboxMessageDiv.scrollHeight - shoutboxMessageDiv.offsetHeight + 100;
			}
		}		
	},
	
	/**
	 * WCF Time Converter 
	 */
	_timeConverter: function(uTimestamp) {  
		var $date = new Date();
        var $timestamp = ($date.getTime() - $date.getMilliseconds()) / 1000;
        var $offset = $timestamp - TIME_NOW;
            
        var $uTimestamp = uTimestamp;
        var $output = "";
        
        // timestamp is less than 60 seconds ago
        if ($uTimestamp >= $timestamp || $timestamp < ($uTimestamp + 60)) {
            $output = WCF.Language.get('wcf.date.relative.now');
        }
        // timestamp is less than 60 minutes ago (display 1 hour ago rather than 60 minutes ago)
        else if ($timestamp < ($uTimestamp + 3540)) {
            var $minutes = Math.max(Math.round(($timestamp - $uTimestamp) / 60), 1);
            $output = WCF.Language.get('wcf.date.relative.minutes', { minutes: $minutes });
        }
        // timestamp is less than 24 hours ago
        else if ($timestamp < ($uTimestamp + 86400)) {
            var $hours = Math.round(($timestamp - $uTimestamp) / 3600);
            $output = WCF.Language.get('wcf.date.relative.hours', { hours: $hours });
        }
        
        return $output;
	},
	
	_sprintf: function sprintf(string, args) {
	    if (!string || !args)
	        return '';
	 
	    var arg, reg;
	 
	    for (var index in args) {
	        arg = args;
	 
	        if (typeof arg == 'string')
	        reg = '%s';
	 
	        else if (typeof arg == 'number' && /\./.test(arg.toString()))
	        reg = '%f';
	 
	        else if (typeof arg == 'number')
	        reg = '%d';
	 
	        else
	            continue;
	 
	        string = string.replace(reg, arg);
	    }
	 
	    return string;
	},
	
	_test: function() { alert('testFunktion'); }
});